
# q-expansion of newform 3680.2.a.t, downloaded from the LMFDB on 03 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3680
weight = 2
poly_data = [-3, 13, -8, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 1, 0, 0], 1], [[1, -6, 1, 1], 1], [[5, -7, 0, 1], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0, 0, 0], [0, 0, 0, -1], [-1, 0, 0, 0], [1, 1, 0, 0], [-3, 0, 1, 0], [0, -2, 1, 0], [-1, 1, -1, 1], [-3, 1, 0, 0], [1, 0, 0, 0], [1, -1, 1, 0], [-2, 2, -3, 0], [-4, -3, 0, 1], [2, 0, 2, 1], [0, 1, 0, 1], [-1, 3, -1, -2], [2, 2, -4, 2], [-4, -4, 0, 2], [1, -2, 1, 4], [-2, -1, -2, 1], [2, -1, 2, -2], [-3, 2, 3, 1], [-4, -1, 4, -1], [4, -6, -2, 4], [4, -1, -4, -1], [-1, -4, 3, 2], [6, 2, 0, 0], [1, -1, -3, 3], [0, 1, -2, 1], [3, 1, -2, -4], [-2, -1, 0, 3], [-9, -1, 5, 0], [-5, 4, -1, -3], [-3, 6, 2, -5], [-5, 6, -7, -1], [3, -2, 0, -3], [-8, 5, -4, -2], [-4, 7, -2, -1], [-10, -1, -1, 1], [2, 0, 2, -2], [7, -3, 1, -3], [-11, -2, 1, -1], [1, -1, 4, 0], [0, -2, 4, 0], [7, -10, 1, 7], [0, 1, -4, -4], [-12, -1, -2, 5], [-8, 0, -2, -2], [-6, 0, 6, -6], [6, 1, 0, 7], [-12, 5, -2, -5], [-1, -4, -3, 1], [3, 2, -3, 1], [-10, 4, -6, -2], [-5, -6, 4, -5], [-3, 0, 3, -5], [5, -2, 1, 4], [5, 3, -3, -4], [9, -6, -6, -1], [-13, 10, -5, -5], [-2, -1, 4, -7], [-2, -6, 0, -2], [8, 6, 2, -8], [7, -3, 5, -3], [1, 2, -1, 3], [-17, 4, 6, -5], [-1, 0, -6, 9], [-1, 6, 5, -5], [7, 1, -4, -8], [1, 5, -9, 1], [-5, -3, 7, 2], [7, 0, -3, 7], [0, 5, -6, -1], [12, -10, -4, 6], [-20, 4, 4, -2], [-1, 1, 3, 5], [4, -9, 0, 11], [5, -5, 9, -1], [22, -7, 2, 6], [8, -8, 8, -2], [6, -3, -3, -7], [-6, -5, 2, 1], [3, 5, -6, 0], [16, -7, -2, 5], [-1, 2, -4, 5], [12, -12, -1, 4], [-4, 17, 3, -7], [-11, 16, 4, -9], [4, 2, 6, -12], [-13, 3, 5, 4], [2, -4, 0, 12], [-12, 4, 0, -6], [0, 4, 2, 2], [15, 5, 5, -6], [3, -6, -5, 9], [-23, 10, -9, -5], [-5, -6, 10, -5], [17, -3, -3, 2], [8, -8, -6, 6], [-8, 1, 6, -5], [11, 9, 1, 4], [-12, 1, -1, -3], [-14, 1, 6, -5], [0, 12, -4, 0], [6, 0, 8, -12], [-3, -9, 1, 11], [19, 0, 7, -7], [12, 10, 0, 9], [-14, 0, 10, -8], [1, 7, -1, 3], [4, -7, 7, 3], [18, 2, 6, -12], [16, 0, -2, -2], [-1, -4, -1, 0], [3, -8, -8, -3], [-36, 3, -4, -3], [-4, 7, -8, 3], [4, -8, -2, -8], [21, -1, 5, 2], [-10, 6, -9, 2], [-28, 8, 0, -4], [21, -7, -3, 7], [5, 4, 1, 7], [2, 8, 0, -4], [8, -9, 1, -1], [14, 2, -4, -4], [-11, -7, -2, 2], [-15, 4, 5, 4], [15, -2, 6, 9], [-13, -7, 7, -1], [-16, 2, -2, -4], [-25, 0, 1, 11], [9, 12, -4, -1], [-16, 3, 12, 1], [12, 5, 8, -7], [-16, 1, -1, -9], [-10, -4, -4, 12], [6, -15, 6, 3], [-2, 18, -6, -2], [-14, -3, 10, 5], [-19, 13, -1, -5], [1, 2, -3, -1], [4, -12, 10, 2], [-19, 5, -7, -12], [6, 4, 10, -4], [12, -6, 2, 0], [12, -1, -2, -5], [-19, 8, 14, -9], [-19, -6, -3, -7], [-23, 8, -3, 1], [13, -7, -1, 4], [13, 5, -9, 1], [4, 6, -4, 2], [7, -8, -8, 11], [7, -7, 1, 12], [12, 1, 6, -11], [-2, -5, 8, -11], [-14, -9, 6, 1], [23, 13, -1, 6], [-7, 11, -2, -4], [-21, -1, 7, -5], [-12, -11, 7, -7], [15, -15, -5, 7], [3, -3, -3, 14], [-11, 9, 5, 3], [-25, -3, 6, 4], [7, 5, 7, -1], [-5, 5, 1, 7], [2, 10, -8, -2], [16, -11, 4, 7], [12, 1, 12, -16], [-12, 1, -4, -11], [3, 9, -1, 8], [-2, -2, 12, 4], [13, -3, 1, -3], [4, 9, -4, -6], [-14, -9, -8, 13], [5, 4, 1, -1], [-9, -5, 5, 6], [-17, 4, -9, 2], [-3, 2, -6, -9], [-5, 9, -15, -2], [5, -4, 13, -6], [42, -2, -9, 2], [-12, -4, -2, -12], [15, -15, -5, 3], [-3, 8, -10, 5], [1, 8, -4, 7], [18, 5, 6, 7], [30, -6, -10, 7], [-23, -5, 7, 1], [-13, 10, 9, 3], [6, 2, 2, 8], [-18, -6, 6, -4], [0, 22, 4, -10], [-26, 6, -4, -10], [-9, 13, 4, -8], [4, 5, -10, 17], [0, -8, 10, -14], [20, -8, -2, -6], [21, -19, -7, 10], [-9, -15, 3, -7], [11, 14, -20, 5], [-6, -7, 10, -14], [-14, 1, 14, 7], [-12, 4, -12, 8], [27, -8, 7, 5], [-20, -7, 10, 3], [8, -14, 16, 3], [32, 3, -15, 5], [-9, -6, 15, -7], [-9, 18, -5, -15], [-9, 11, -17, 0], [-43, 1, -8, 8], [-12, 4, 10, 4], [36, 2, -8, -2], [-14, -1, 0, 5], [-7, 5, 7, -12], [15, -3, 3, 18], [25, 0, -6, -9], [3, -2, -9, 7], [-1, 13, 3, 0], [0, -9, -6, 5], [-20, -4, 4, 5], [-25, -7, -4, 10], [5, 1, -9, -10], [-15, -17, 1, 8], [-9, -13, 14, 2], [18, 2, -3, -8], [-15, 16, 2, -9], [-32, 16, 4, -4], [13, 4, -2, 3], [-21, -10, 11, 0], [38, -4, 2, -6], [-3, 15, -1, -1], [-2, 10, 0, -4], [16, -17, 8, 15], [-2, -6, 4, 4], [26, -1, -16, -3], [-5, 8, -5, -5], [-16, -11, -4, 0], [2, -24, -4, 14], [1, -8, -11, 11], [-33, -13, 11, 0], [-3, -7, 11, 0], [24, -8, 6, 0], [-18, 0, -5, -6], [-43, 9, 5, -6], [27, -6, -14, 7], [13, -1, 9, 1], [1, 5, 9, -1], [6, -6, 14, 2], [-16, 10, -20, 12], [-3, -14, -6, 17], [-28, 11, 12, -1], [-4, -4, 14, -14], [-19, -2, -5, 21], [-13, 16, 11, 2], [8, -16, -12, 10], [28, -4, -15, 10], [-9, 0, -5, 9], [-9, -13, 7, 3], [-24, 5, -2, 11], [-16, 4, 4, -12], [0, 7, -2, 15], [8, 4, -10, -14], [-3, 2, -11, -2], [-27, 14, 5, -12], [40, 9, -10, 3], [-12, 17, 0, -13], [22, 14, -11, 6], [3, 17, -13, 4], [6, 18, -6, -12], [45, 15, -8, 4], [-14, -1, -2, -3], [-19, -12, 17, 3], [-67, 1, 3, -4], [-27, 19, 7, -7], [2, -16, -4, 22], [-2, -16, 0, -4], [-3, -8, 4, -1], [29, 15, 3, 0], [-22, 0, -14, 10], [-9, 10, -1, 4], [-22, -6, 4, 10], [13, -12, -10, 1], [-3, -1, 7, -11], [-18, 0, -8, -4], [6, 21, -14, -3], [-45, -1, 17, -3], [9, -9, 3, 13], [-10, 6, 8, -2], [10, -30, -10, 22], [16, -12, -12, -2], [3, 25, -4, -6], [0, -15, -4, -2], [13, 19, 5, 9], [10, -4, -20, -6], [-24, 1, 8, 10], [-23, -22, 9, -3], [6, -16, 2, 9], [-15, 5, 12, -14], [6, 7, 4, 1], [-10, -16, 2, -16], [-43, -1, 11, -8], [-1, -20, -12, 9], [-3, 12, -13, -1], [-15, -1, -9, -15], [21, 4, -10, 13], [32, 16, -6, 8], [-1, -3, 17, 5], [-25, 13, -17, -8], [-43, -9, 0, 4], [-6, 0, -10, -4], [12, -12, 16, 11], [-7, 9, 6, 4], [-10, -9, -4, 9], [-3, 6, 1, -7], [0, 8, -23, -4], [4, 14, -2, -8], [-18, 14, -6, 3], [-19, -14, 13, 4], [3, -2, 3, 4], [21, -15, 21, 12], [-24, -12, 18, -8], [-43, -1, 21, -7], [-12, -10, 0, 22], [-37, 16, 10, -11], [30, 15, -17, 5], [7, 11, -6, 12], [6, 8, 0, 6], [3, -15, 9, -6], [-10, -11, -6, 28], [31, -16, 13, -5], [12, 5, -15, 17], [-52, 1, 12, -7], [-12, 1, 17, 13], [13, -1, 9, -15], [-18, -1, -12, 15], [0, -4, 16, -11], [-9, -6, -9, 10], [3, -26, 7, 11], [-11, 0, 17, 6], [-20, 12, -4, 0], [24, -16, 6, 8], [26, -14, 0, 14], [16, -8, -20, -16], [-1, -17, 23, 0], [29, 10, 0, -9], [-26, -6, 0, 10], [-12, 12, -6, -24], [23, 20, -5, -9], [21, 18, 1, -11], [28, -4, -28, 6], [-39, 19, -3, -15], [28, -6, -22, 8], [-40, -12, -2, -12], [38, -18, 14, 18], [-61, 0, -6, 1], [-29, -2, 11, -17], [42, -8, -8, 4], [-76, 2, -4, 0], [25, -1, -7, 10], [-53, -1, -3, -8], [-35, -19, 17, -5], [-2, -4, -6, -17], [55, -12, -9, 13], [32, -19, 20, 4], [-6, 23, -6, 1], [-8, -26, -2, 10], [-49, 2, -1, -5], [22, -6, -2, 18], [51, -11, 3, 6], [9, 7, 1, -21], [-40, -2, 4, -10], [-12, 6, -9, 12], [-41, 6, 15, 9], [42, 31, -10, 11], [31, -5, -5, 5], [-22, -2, 28, -16], [17, 6, 11, -17], [-33, -9, -13, 25], [-77, -6, 3, 2], [0, 36, -4, -6], [5, 4, 9, -13], [18, -7, 14, 27], [49, 1, -1, -18], [34, -12, 17, 10], [7, -2, -10, 19], [30, -10, 12, 2], [-3, 6, -12, -23], [-4, -16, 18, -6], [-37, 28, 0, -15], [-19, 14, 7, 15], [-66, -8, 2, 10], [-18, -5, -4, -13], [39, -14, -20, -3], [22, 13, -16, 9], [-12, 35, 4, -21], [-41, -12, 8, -15], [68, 2, 6, 4], [19, 11, 9, -24], [-33, 14, -19, 7], [-16, 13, -16, 17], [7, 30, 2, -3], [-15, 19, -5, -20], [-26, 11, 18, -20], [-38, 3, -4, -3], [30, 19, -3, 3], [-14, -1, 8, -3], [4, 20, -12, 12], [73, -16, -9, 7], [18, 7, 14, 0], [10, -21, 26, 9], [-5, 33, 3, -29], [-3, -15, 10, -14], [-37, -13, 3, -16], [15, 0, 10, -17], [-32, 2, -6, 2], [17, 0, 9, 2], [49, -13, -12, 2], [-11, 6, 1, 14], [-24, -15, -2, -15], [-20, -8, 14, 8], [-18, -8, 16, -18], [-21, 16, 11, 7], [-5, -20, 7, 13], [46, 7, 6, -11], [-19, 4, -6, 7], [39, -10, 13, 11], [-4, 1, 28, -20]]
